@@ -14,8 +14,8 @@
  *  GNU Library General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  along with this program; if not, write to the Free Software Foundation,
+ *  Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
  */
 
 #ifndef _XFCE_BACKDROP_H_
@@ -40,22 +40,37 @@ typedef struct _XfceBackdropPriv XfceBackdropPriv;
 
 typedef enum
 {
-    XFCE_BACKDROP_IMAGE_AUTO = 0,
+    XFCE_BACKDROP_IMAGE_INVALID = -1,
+    XFCE_BACKDROP_IMAGE_NONE = 0,
     XFCE_BACKDROP_IMAGE_CENTERED,
     XFCE_BACKDROP_IMAGE_TILED,
     XFCE_BACKDROP_IMAGE_STRETCHED,
     XFCE_BACKDROP_IMAGE_SCALED,
     XFCE_BACKDROP_IMAGE_ZOOMED,
+    XFCE_BACKDROP_IMAGE_SPANNING_SCREENS,
 } XfceBackdropImageStyle;
 
 typedef enum
 {
+    XFCE_BACKDROP_COLOR_INVALID = -1,
     XFCE_BACKDROP_COLOR_SOLID = 0,
     XFCE_BACKDROP_COLOR_HORIZ_GRADIENT,
     XFCE_BACKDROP_COLOR_VERT_GRADIENT,
     XFCE_BACKDROP_COLOR_TRANSPARENT,
 } XfceBackdropColorStyle;
 
+typedef enum
+{
+    XFCE_BACKDROP_PERIOD_INVALID = -1,
+    XFCE_BACKDROP_PERIOD_SECONDS = 0,
+    XFCE_BACKDROP_PERIOD_MINUES,
+    XFCE_BACKDROP_PERIOD_HOURS,
+    XFCE_BACKDROP_PERIOD_STARTUP,
+    XFCE_BACKDROP_PERIOD_HOURLY,
+    XFCE_BACKDROP_PERIOD_DAILY,
+    XFCE_BACKDROP_PERIOD_CHRONOLOGICAL,
+} XfceBackdropCyclePeriod;
+
 struct _XfceBackdrop
 {
     GObject gobject;
@@ -71,6 +86,7 @@ struct _XfceBackdropClass
     /*< signals >*/
     void (*changed)(XfceBackdrop *backdrop);
     void (*cycle)(XfceBackdrop *backdrop);
+    void (*ready)(XfceBackdrop *backdrop);
 };
 
 GType xfce_backdrop_get_type             (void) G_GNUC_CONST;
@@ -100,10 +116,6 @@ void xfce_backdrop_set_second_color      (XfceBackdrop *backdrop,
 void xfce_backdrop_get_second_color      (XfceBackdrop *backdrop,
                                           GdkColor *color);
 
-void xfce_backdrop_set_show_image        (XfceBackdrop *backdrop,
-                                          gboolean show_image);
-gboolean xfce_backdrop_get_show_image    (XfceBackdrop *backdrop);
-
 void xfce_backdrop_set_image_style       (XfceBackdrop *backdrop,
                                           XfceBackdropImageStyle style);
 XfceBackdropImageStyle xfce_backdrop_get_image_style
@@ -111,32 +123,35 @@ XfceBackdropImageStyle xfce_backdrop_get_image_style
 
 void xfce_backdrop_set_image_filename    (XfceBackdrop *backdrop,
                                           const gchar *filename);
-G_CONST_RETURN gchar *xfce_backdrop_get_image_filename
+const gchar *xfce_backdrop_get_image_filename
                                          (XfceBackdrop *backdrop);
 
-void xfce_backdrop_set_list              (XfceBackdrop *backdrop,
-                                          gchar *backdrop_list);
-G_CONST_RETURN gchar *xfce_backdrop_get_list
-                                         (XfceBackdrop *backdrop);
-
-void xfce_backdrop_set_brightness        (XfceBackdrop *backdrop,
-                                          gint brightness);
-gint xfce_backdrop_get_brightness        (XfceBackdrop *backdrop);
-
-void xfce_backdrop_set_saturation        (XfceBackdrop *backdrop,
-                                          gdouble saturation);
-gdouble xfce_backdrop_get_saturation     (XfceBackdrop *backdrop);
-
 void xfce_backdrop_set_cycle_backdrop    (XfceBackdrop *backdrop,
                                           gboolean cycle_backdrop);
 gboolean xfce_backdrop_get_cycle_backdrop(XfceBackdrop *backdrop);
 
+void xfce_backdrop_set_cycle_period      (XfceBackdrop *backdrop,
+                                          XfceBackdropCyclePeriod period);
+XfceBackdropCyclePeriod xfce_backdrop_get_cycle_period
+                                         (XfceBackdrop *backdrop);
+
 void xfce_backdrop_set_cycle_timer       (XfceBackdrop *backdrop,
                                           guint cycle_timer);
 guint xfce_backdrop_get_cycle_timer      (XfceBackdrop *backdrop);
 
+void xfce_backdrop_set_random_order      (XfceBackdrop *backdrop,
+                                          gboolean random_order);
+gboolean xfce_backdrop_get_random_order  (XfceBackdrop *backdrop);
+
+void xfce_backdrop_force_cycle           (XfceBackdrop *backdrop);
+
+
 GdkPixbuf *xfce_backdrop_get_pixbuf      (XfceBackdrop *backdrop);
 
+void xfce_backdrop_generate_async        (XfceBackdrop *backdrop);
+
+void xfce_backdrop_clear_cached_image    (XfceBackdrop *backdrop);
+
 G_END_DECLS
 
 #endif
